--[[
    WarcraftPets by LordFarlander
    Version 1.0.6
    $Revision: 2 $
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

WarcraftPets = LibStub( "AceAddon-3.0" ):NewAddon( "WarcraftPets", "AceEvent-3.0" );
WarcraftPets.version = { major = 1, minor = 0, revision = tonumber( "6" ) };

WarcraftPets.cache = {};

function WarcraftPets:OnInitialize()
    local defaults = {
        global = {
            companions = {};
        },
    };

    self.db = LibStub( "AceDB-3.0" ):New( "WarcraftPetsDB", defaults );
    self.db.global.version = self.version;
end--WarcraftPets:OnInitialize()

function WarcraftPets:OnEnable()
    self:RegisterEvent( "BAG_UPDATE",
        function( _, bagID )
            if( bagID ) then
                WarcraftPets:DoCheck( "BAGS", bagID, bagID );
            end--if
        end );
    self:RegisterEvent( "BANKFRAME_OPENED",
        function()
            WarcraftPets:DoCheck( "BAGS", BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS );
        end );
    self:RegisterEvent( "COMPANION_UPDATE",
        function( _, type )
            if( not type ) then
                WarcraftPets:DoCheck( "COMPANIONS" );
            end--if
        end );
    self:RegisterEvent( "COMPANION_UPDATE",
        function( _, type )
            if( not type ) then
                WarcraftPets:DoCheck( "COMPANIONS" );
            end--if
        end );
    self:RegisterEvent( "UNIT_INVENTORY_CHANGED",
        function( _, unit )
            if( unit == "player" ) then
                WarcraftPets:DoCheck( "INVENTORY" );
            end--if
        end );
    self:DoCheck( "COMPANIONS" );
end--WarcraftPets:OnEnable()

function WarcraftPets:DoCheck( type, min, max )
    self.db.global.companions = {};

    local companions = self.db.global.companions;
    local PetAndMountDatabase = LibStub( "LibPetAndMountDatabase-1.1", true );    
    local ItemInSet = PetAndMountDatabase.ItemInSet;
    local cache = self.cache;
    local list;

    if( type == "BAGS" ) then
        for bag = min, max do
            cache[bag] = {};
            for slot = 1, GetContainerNumSlots( bag ) do
                local itemLink = GetContainerItemLink( bag, slot );

                list = cache[bag];
                if( itemLink ) then
                    local itemID = tonumber( itemLink:match( "(%d+)" ) );

                    if( ItemInSet( PetAndMountDatabase, itemID, "Critters" ) and ( not ( ItemInSet( PetAndMountDatabase, itemID, "Critters.Quest" ) or ItemInSet( PetAndMountDatabase, itemID, "Critters.Children" ) ) ) ) then
                        tinsert( list, itemID );
                    end--if
                 end--if
            end--for
        end--for
    end--if
    if( type == "COMPANIONS" ) then
        cache[type] = {};
        if( PetAndMountDatabase.GetPlayersCompanions ) then
            list = cache[type];

            for id, critter in pairs( PetAndMountDatabase:GetPlayersCompanions( "CRITTER" ) ) do
                tinsert( list, critter.spellID * -1 );
            end--for
        end--if
    end--if
    if( type == "INVENTORY" ) then
        cache[type] = {};
    
        local iterator, handle = PetAndMountDatabase:IterateSet( "Critters.Equipment" );
        local pet, equipmentSlot = iterator( handle );

        list = cache[type];
        while pet do
            equipmentSlot = GetInventorySlotInfo( equipmentSlot );
            
            local itemLink = GetInventoryItemLink( "player", equipmentSlot );

            if( itemLink and ( tonumber( string.match( itemLink, "(%d+)" ) == pet ) ) ) then
                tinsert( list, pet );
            end--if
            pet, equipmentSlot = iterator( handle );
        end--for
    end--if
    for _, set in pairs( cache ) do
        for _, id in pairs( set ) do
            tinsert( companions, id );
        end--for
    end--for
end--WarcraftPets:DoCheck()
